/**
 *
 * Copyright 2013 Vibes.IO
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = (function() {
    var Q = require('q'),
        check = require('validator').check,
        Signer = require('./signer'),
        moment = require('moment'),
        Url = require('url'),
        util = require('util'),
        _ = require('lodash'),
        ResourceSet = require('./resource_set'),
        request = require('request');

    /**
     * @class Space
     * @description Abstract all resource management operations on a space. You may use instances of this class exactly as if
     *  they were a local database. In the browser, you get access to an advanced faceted databases to support your rich HTML5 apps.
     */
    Space = (function() {

        const META_RESOURCE_SET = "resourceSets";

        // Our internal cache of resolved resource sets
        var resourceSets = {};

        /**
         *
         * @param url
         * @param spec
         * @param options
         * @constructor
         */
        function Space(url, spec, options) {
            this.token = spec.token;
            this.secret = spec.secret;
            this.key = spec.key;
            this.url = Url.parse(url);
            this.options = options || {};
        }

        /**
         *
         * @param rsetKey
         * @param options
         */
        Space.prototype.resourceSet = function(rsetKey, options) {
            if(!resourceSets[rsetKey]) resourceSets[rsetKey] = this.resolve(META_RESOURCE_SET, rsetKey, ResourceSet);
            return resourceSets[rsetKey];
        };

        /**
         * Resolve a generic resource from this space using the configured contract tokens
         *
         * @param rsetKey
         * @param key
         * @param Impl
         * @param options
         */
        Space.prototype.resolve = function(rsetKey, key, Impl, options) {
            options = options || {};

            //TODO: Check our local cache

            // Execute a resolve operation in the context of this space
            return new Impl(this, key, this.operation(rsetKey, key), options);
        };

        Space.prototype.buildResourceUrl = function(rsetKey, key) {
            var url = util.format("%s//%s/%s", this.url.protocol, this.url.host, rsetKey);
            if(key)
                url += "/" + key;
            return url;
        };

        Space.prototype.operation = function(rsetKey, key, options) {
            var _this = this;
            var defer = Q.defer();
            options = options || {};

            Q.nextTick(function() {

                // Build the url
                var url = _this.buildResourceUrl(rsetKey, key);

                var headers = {
                    'x-vibes-timestamp':moment().utc().unix()
                };

                headers['x-vibes-space'] = _this.key;
                headers = _.extend(headers, options.headers || {});

                // Sign the request if we have a valid token
                if(_this.token) {

                    var signer = new Signer({
                        key:_this.token,
                        token:_this.secret
                    });

                    // Special version to sign a full url
                    var sign = signer.signUrl(url, undefined, headers);
                    headers['authorization'] = signer.buildAuthorizationHeader(sign);
                }

                var req = {
                    uri:url,
                    method:options.method || 'GET',
                    json:true,
                    headers:headers
                };

                if(options.data) {
                    req.body = options.data
                }

                // Send the request to the Vibes node
                request(req, function(err, resp, body) {
                    if(err) defer.reject({error:err});
                    else {
                        if(resp.statusCode >= 400) {
                            defer.reject({error:'network-error', status:resp.statusCode});
                        }
                        else {
                            defer.resolve(body);
                        }
                    }

                });
            });

            return defer.promise;
        };

        return Space;
    })();

    /**
     * Vibes.space implementation
     *
     * Will retrieve the space spec from the supplied resource url and instantiate a
     * space object to manipulate resources.
     */
    return function(url, options) {
        var defer = Q.defer();
        options = options || {};

        // Override module dependencies to help in unit testing
        if(options.modules) {
            request = options.modules.request || request;
            moment = options.modules.moment || moment;
        }

        Q.nextTick(function() {
            try {
                // Check the url
                check(url, "error-invalid-url").isUrl();

                // Setup any headers
                var headers = {};
                if(options.headers) {
                    headers = options.headers;
                }

                // If we have a key, we must sign this request
                if(options.key) {

                    // Sign the request
                    var signer = new Signer({
                        key:options.key,
                        token:options.token
                    });

                    var headers = {
                        'x-vibes-timestamp':moment().utc().unix()
                    };

                    if(options.targetSpaceKey) {
                        headers['x-vibes-space'] = options.targetSpaceKey;
                    }

                    // Special version to sign a full url
                    var sign = signer.signUrl(url, undefined, headers);
                    headers['authorization'] = signer.buildAuthorizationHeader(sign);
                }

                // Send the request to the Vibes node
                request({uri:url, method:'GET', json:true, headers:headers}, function(err, resp, body) {
                    if(err) defer.reject({error:err});
                    else {
                        if(resp.statusCode === 200 && body) {
                            defer.resolve(new Space(url, body, options));
                        }
                        else
                            defer.reject({error:'network-error', status:resp.statusCode});
                    }
                });

            }
            catch(err) {
                defer.reject(err);
            }

        });

        return defer.promise;
    };

})();
