/**
 *
 * Copyright 2013 Vibes.IO
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var Vibes = require('../.');

const TEST_SPACE_URI = "https://system.vibes.io/spaces/system";

describe('when calling space', function() {

    describe('general', function() {
        var overridenModules = {
            request: function(req, fn) {
                fn(undefined, {statusCode:200}, {
                    key:'test-space'
                });
            }
        };

        it('with an invalid url', function(done) {
            Vibes.space("test-invalid-url", {modules:overridenModules}).then(function(space) {
                expect(space).toBeUndefined();
                done();
            }, function(err) {
                expect(err).toEqual({ name : 'ValidatorError', message : 'error-invalid-url' });
                done();
            });
        });
    });

    describe('buildResourceUrl', function(){
        var overridenModules = {
            request: function(req, fn) {
                fn(undefined, {statusCode:200}, {
                    key:'test-space'
                });
            }
        };

        it('should produce "http://system.vibes.io/spaces/system" if given spaces and system has parameter', function(done){
            Vibes.space(TEST_SPACE_URI, {modules:overridenModules}).then(function(space) {
                var url = space.buildResourceUrl('spaces', 'system');
                expect(url).toEqual("https://system.vibes.io/spaces/system");
                done();
            });
        });

        it('should produce "http://system.vibes.io/spaces" if no resource key is provided', function(done) {
            Vibes.space(TEST_SPACE_URI, {modules:overridenModules}).then(function(space) {
                var url = space.buildResourceUrl('spaces');
                expect(url).toEqual("https://system.vibes.io/spaces");
                done();
            });
        });

    });

    describe('operation', function() {
        var overridenModules = {
            request: function(req, fn) {
                fn(undefined, {statusCode:200}, {
                    key:'test-space',
                    token:"0123456789",
                    secret: "012232321312321"
                });
            },
            moment:function(){
                return {
                    utc:function() {
                        return {
                            unix:function() {
                                return 1;   // test timestamp
                            }
                        }
                    }
                }
            }
        };

        beforeEach(function() {
            spyOn(overridenModules, 'request').andCallThrough();
        });

        it('should produce a valid get_resource operation', function(done) {

            Vibes.space(TEST_SPACE_URI, {modules:overridenModules}).then(function(space) {
                var promise = space.operation('spaces','system');
                expect(promise).toBeDefined();
                promise.then(function(result) {
                    expect(result).toBeDefined();
                    expect(result.key).toBe('test-space');
                    expect(overridenModules.request).toHaveBeenCalled();
                    expect(overridenModules.request.argsForCall[1][0].uri).toEqual('https://system.vibes.io/spaces/system');
                    expect(overridenModules.request.argsForCall[1][0].method).toEqual('GET');
                    expect(overridenModules.request.argsForCall[1][0].json).toBeTruthy();
                    expect(overridenModules.request.argsForCall[1][0].headers).toBeDefined();
                    expect(overridenModules.request.argsForCall[1][0].headers['x-vibes-timestamp']).toBe(1);
                    expect(overridenModules.request.argsForCall[1][0].headers.authorization).toBe('Vibes-Signature token="012232321312321", algo="sha1", signature="35f7dd45319d08e59cac7c8d21bff8d147595017"')
                    expect(overridenModules.request.argsForCall[1][0].headers['x-vibes-space']).toBe("test-space");
                    done();
                }, done);
            });
        });
    });

});
